package leelib.graphics
{
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.DisplayObject;
	import flash.display.Graphics;
	import flash.display.Shape;
	import flash.display.Sprite;
	import flash.filters.GradientBevelFilter;
	import flash.geom.Matrix;
	import flash.geom.Point;
	import flash.geom.Rectangle;
	import flash.utils.ByteArray;
	
	import leelib.appropriated.JPEGEncoder;
	import leelib.appropriated.Scale9BitmapSprite;

	public class GrUtil
	{
		public static function makeRect($width:Number, $height:Number, $color:uint=0xff0000, $alpha:Number=1):Sprite
		{
			var s:Sprite = new Sprite();
			s.graphics.beginFill($color, $alpha);
			s.graphics.drawRect(0,0, $width, $height);
			s.graphics.endFill();
			return s;
		}

		public static function makeRoundRectRect($width:Number, $height:Number, $radius:Number, $color:uint=0xff0000, $alpha:Number=1):Sprite
		{
			var s:Sprite = new Sprite();
			s.graphics.beginFill($color, $alpha);
			s.graphics.drawRoundRect(0,0, $width, $height, $radius, $radius);
			s.graphics.endFill();
			return s;
		}

		public static function replaceRect($shape:Sprite, $width:Number, $height:Number, $color:uint=0xff0000, $alpha:Number=1):void
		{
			$shape.graphics.clear();
			$shape.graphics.beginFill($color, $alpha);
			$shape.graphics.drawRect(0,0, $width, $height);
			$shape.graphics.endFill();
		}
		
		public static function makeBitmap($width:Number, $height:Number, $color32:uint=0xffff8888):Bitmap
		{
			var bmd:BitmapData = new BitmapData($width,$height,true,$color32);
			return new Bitmap(bmd);
		}
		
		public static function makePerlinBitmap($width:Number, $height:Number, $bw:Boolean=true):Bitmap
		{
			var bmd:BitmapData = new BitmapData($width,$height,false,0xffffffff);
			bmd.perlinNoise(55,55,2, int(Math.random()*0xffffffff),false,false,2,$bw);
			return new Bitmap(bmd);
		}
		
		public static function scale9BitmapData($b:BitmapData, $newWidth:Number, $newHeight:Number, $padding:Number):BitmapData
		{
			var s:Sprite = new Scale9BitmapSprite($b, new Rectangle($padding, $padding, $b.width-$padding*2, $b.height-$padding*2));
			s.width = $newWidth;
			s.height = $newHeight;
			var b:BitmapData = new BitmapData($newWidth, $newHeight, true, 0x0);
			b.draw(s);
			return b;
		}
		
		public static function cropBitmapData($b:BitmapData, $w:int, $h:int):BitmapData
		{
			var b:BitmapData = new BitmapData($w,$h, true, 0x0);
			b.draw($b);
			return b;
		}
		
		public static function localToLocal(from:DisplayObject, to:DisplayObject):Point 
		{
			return to.globalToLocal(from.localToGlobal(new Point()));
		};
		
		public static function fitInRect($d:DisplayObject, $r:Rectangle, $fillInRectInstead:Boolean=false):void
		{
			// xxx refactor with "rectFit" below
			
			var offx:Number = 0;
			var offy:Number = 0;
			
			var isTaller:Boolean = $r.width/$r.height > $d.width/$d.height;
			
			if ((isTaller && ! $fillInRectInstead) || (! isTaller && $fillInRectInstead)) {
				$d.height = $r.height;
				$d.scaleX = $d.scaleY;
				offx = ($r.width - $d.width) / 2;
			}
			else {
				$d.width = $r.width;
				$d.scaleY = $d.scaleX;
				offy = ($r.height - $d.height) / 2;
			}
			$d.x = $r.x + offx;
			$d.y = $r.y + offy;
		}

		/*
		public static function rectFit($srcWidth:Number, $srcHeight:Number, $destWidth:Number, $destHeight:Number, $fillInRectInstead:Boolean=false):Rectangle
		{
			REFACTOR THIS
		
			var offx:Number = 0;
			var offy:Number = 0;
			
			var isTaller:Boolean = $r.width/$r.height > $d.width/$d.height;
			
			if ((isTaller && ! $fillInRectInstead) || (! isTaller && $fillInRectInstead)) {
				$d.height = $r.height;
				$d.scaleX = $d.scaleY;
				offx = ($r.width - $d.width) / 2;
			}
			else {
				$d.width = $r.width;
				$d.scaleY = $d.scaleX;
				offy = ($r.height - $d.height) / 2;
			}
			$d.x = $r.x + offx;
			$d.y = $r.y + offy;
		}
		*/
		
		public static function bitmapDataToJpeg($b:BitmapData, $quality:int):ByteArray
		{
			var j:JPEGEncoder = new JPEGEncoder($quality);
			return j.encode($b);
		}
		
		public static function makeCrossHatchBitmapData($color1:uint=0xffffffff, $color2:uint=0xff000000):BitmapData
		{
			var b:BitmapData = new BitmapData(2,2,true,0x0);
			b.setPixel32(0,0, $color1);
			b.setPixel32(1,1, $color1);
			b.setPixel32(0,1, $color2);
			b.setPixel32(1,0, $color2);
			return b;
		}
		
		public static function makeCrossHatchRect($width:Number, $height:Number, $color1:uint=0xffffffff, $color2:uint=0xff000000):Sprite
		{
			var b:BitmapData = makeCrossHatchBitmapData($color1,$color2);
			
			var s:Sprite = new Sprite();
			s.graphics.beginBitmapFill(b, null, true, false);
			s.graphics.drawRect(0,0,$width,$height);
			s.graphics.endFill();
			
			return s;
		}
		
		public static function replaceCrossHatchRect($sprite:Sprite, $width:Number, $height:Number, $color1:uint=0xffffffff, $color2:uint=0xff000000):void
		{
			var b:BitmapData = makeCrossHatchBitmapData($color1,$color2);
			
			$sprite.graphics.clear();
			$sprite.graphics.beginBitmapFill(b, null, true, false);
			$sprite.graphics.drawRect(0,0,$width,$height);
			$sprite.graphics.endFill();
		}
		
		/**
		 * Fills in a BitmapData of dimensions $w,$h with the graphics from $bitmap, 
		 * cropping what sticks out vertically or horizontally.
		 * 
		 * Unless $fitInstead is true, in which case...
		 */
		public static function fillBitmapDataInto($bitmapData:BitmapData, $w:Number,$h:Number, $fitInsteadOfFill:Boolean=false):BitmapData
		{
			var wSrc:Number = $bitmapData.width;
			var wDest:Number = $w;
			var hSrc:Number = $bitmapData.height;
			var hDest:Number = $h;
			var arSrc:Number = wSrc/hSrc;
			var arDest:Number = wDest/hDest;
			
			var b:BitmapData = new BitmapData(wDest,hDest,true,0x0);
			
			var sca:Number;
			
			var offX:Number = 0;
			var offY:Number = 0;
			
			if (! $fitInsteadOfFill) 
			{
				if (arSrc > arDest) { 
					sca = hDest / hSrc;
				}
				else {
					sca = wDest / wSrc;
				}
			}
			else
			{
				if (arSrc > arDest) { 
					sca = wDest / wSrc;
				}
				else {
					sca = hDest / hSrc;
				}
			}
				
			offX = ((wSrc * sca) - wDest) / -2;
			offY = ((hSrc * sca) - hDest) / -2;
			
			var m:Matrix = new Matrix();
			m.scale(sca,sca);
			m.tx = offX;
			m.ty = offY;
			b.draw($bitmapData, m, null,null,null, true);
			
			return b;
		}
	}
}